/*
 * FormMapScript.js
 *
 * Contains all the general scripts used by the form map page
 *
 * @author Rafel Bordas
 *
 */

//sets the namespace to use when creating SVG components.
var SVG_NS ="http://www.w3.org/2000/svg";
var XLINK_NS ="http://www.w3.org/1999/xlink";

var navigationDiagram;
var xmlhttp = new XMLHttpRequest();

var ViewXMLHttp = new XMLHttpRequest();
var viewNodeList = null;
var viewString = "L1";
var sourceViewXstore = "blueprints";
var sourceViewCollection = "generic/mvc/generic/L1_pattern_pool";
var sourceViewDocument = "View.xml";

var ControllerXMLHttp = new XMLHttpRequest();
var controllerNodeList = null;
var controllerString = "L2";
var sourceControllerXstore = "blueprints";
var sourceControllerCollection = "generic/mvc/generic/L2_pattern_pool";
var sourceControllerDocument = "Controller.xml";

var ModelXMLHttp = new XMLHttpRequest();
var modelNodeList = null;
var modelString = "L3";

var SelectedNodeXMLHttp = new XMLHttpRequest();
var selectedNode_nodeList = null;
var selectedNodeString = "selectedNode";
var sourceSelectedNodeXstore = "blueprints";
var sourceSelectedNodeCollection = "generic/mvc/generic/L2_pattern_pxool";

var targetViewXstore = "";
var targetViewCollection = "";
var targetControllerXstore = "";
var targetControllerCollection = "";
var targetModelXstore = "";
var targetModelCollection = "";

var defaultsMode;
var originalControlColour;
var serviceLinked = true;
var emptyTransform;
var messageTransform;

var _newHandler = '';

/** Set up an array of all the names that the user cannot use for element names.
 *  Should keep this in sync with the list in formStructureScripts.js and the configGenerator.xsl */
var invalidNames = new Array();
invalidNames[invalidNames.length] = 'submit';
invalidNames[invalidNames.length] = 'action';
invalidNames[invalidNames.length] = 'new';
invalidNames[invalidNames.length] = 'try';
invalidNames[invalidNames.length] = 'do';
invalidNames[invalidNames.length] = 'if';
invalidNames[invalidNames.length] = 'delete';

window.onload = init;

function init(callCount)
{
    var viewEmbed = document.getElementById('view');

    //make sure we can succesfully access the SVG docuemnt before continuing
    if ((viewEmbed == null) || (viewEmbed.getSVGDocument() == null))
    {
        if ((typeof(callCount) == 'undefined') || isNaN(callCount))
            callCount = 0;
        callCount++;
        if (callCount < 50)
            window.setTimeout('init('+callCount+')', 50);
        else
            alert('Unable to initialise the display.  Please contact Hyfinity.\n\nDetails: Timeout waiting for SVG to init.');
        return;
    }

    navigationDiagram = new NavigationDiagram("navigationDiagram", viewEmbed, 'applicationXML');
    targetViewXstore = fm_view_xstore;
    targetControllerXstore = fm_controller_xstore;
    targetViewCollection = fm_view_collection;
    targetControllerCollection = fm_controller_collection;
    navigationDiagram.setMode('select');
    navigationDiagram.draw();

    if (navigationDiagram._defaults._skin != '')
        defaultsMode = 'visible';
    else
        defaultsMode = 'hidden';
    toggleDefaults();

    emptyTransform =  new SimpleTransform('xsl/empty_details.xsl', document.getElementById('context_panel'));
    messageTransform = new SimpleTransform('xsl/empty_details.xsl', document.getElementById('message_context_panel'));

    if (document.result_form.gen_control != null)
    {
        //alert(document.result_form.gen_control_type.value)
        if (document.result_form.gen_control_type.value == 'Page')
            navigationDiagram.setSelected(navigationDiagram._pages[document.result_form.gen_control.value]);
        else if (document.result_form.gen_control_type.value == 'Handler')
            navigationDiagram.setSelected(navigationDiagram._handlers[document.result_form.gen_control.value]);
        else
            navigationDiagram.setSelected(navigationDiagram._links[document.result_form.gen_control.value]);
    }
}

/**********************

  * Utility functions, used by this script and most of the scripts within the containing directory

***********************/

/**
 * Checks to see if a name is exists amongst an objects property names
 *
 * @param name The name to check for
 * @param object The object whose properties are to be checked
 * @returns true if found, false if not
 */
function checkPropertyNameExists(name, object)
{
    for(property in object)
    {
        if(property == name)
        {
            alert('Name already in use. Please specify another unique name.');
            return true;
        }
    }
    return false;
}

/*
 * Similar to above, but searches within arrays
 */
function arrayContains(array, value)
{
    var found = false;
    for (var i = 0; i < array.length; ++i)
    {
        if (array[i] == value)
        {
            found = true;
            break;
        }
    }
    return found;
}

/* Checks for invalid entry characters */
function checkInvalidChars(id)
{
    var filter = /[\W]/;
    if((id == '') || (filter.test(id)) || !isNaN(id.charAt(0)))
    {
        alert('Names cannot be blank, include non-word characters, or start with a number. Please specify a new unique name.');
        return true;
    }
    else
        return false;
}

/**
 * Convenience method to check for invalid name entries
 */
function checkInvalidNames(name)
{
    var nameInvalid = false;
    if(arrayContains(invalidNames, name))
    {
        nameInvalid = true;
        alert("The name of the element you have chosen is invalid.  Please choose another.");
    }
    return nameInvalid;
}

/**
 * Converts any of the five XMl entities in the given string to
 * their escaped versions.
 * @param XMLString The string to check for the XML characters.
 */
function escapeXMLCharacters(XMLString)
{
    if ((XMLString != null) || (typeof(XMLString) != 'undefined'))
    {
        XMLString = XMLString.replace(/&/gi , "&amp;");
        XMLString = XMLString.replace(/</gi , "&lt;");
        XMLString = XMLString.replace(/>/gi , "&gt;");
        XMLString = XMLString.replace(/'/gi , "&apos;");
        XMLString = XMLString.replace(/"/gi , "&quot;");
        return XMLString;
    }
}

/**
 * Clones a (packed) array
 * This only performs a shallow clone, array elements are copied by reference
 */
function cloneArray(array)
{
    var newArray = new Array();
    for(var i = 0; i < array.length; ++i)
    {
        newArray[i] = array[i];
    }
    return newArray;
}

/**
 * Deletes the given element from an array
 *
 * @param element The element to remove
 * @param array The array to alter
 */
function removeElement(element, array)
{
    var arrayIndex = 0;
    for(var i = 0; i < array.length; ++i)
    {
        if(array[i] == element)
        {
            arrayIndex = i;
        }
    }

    array.splice(arrayIndex, 1);
    return array
}

function populateXMLNodeList(xmlhttpString, hasSubsectionHandler,status)
{

    //only do this if there is a selected handler
    if(navigationDiagram._selected != null && (navigationDiagram._selected.constructor == Handler
        || navigationDiagram._selected.constructor == Link
        || (navigationDiagram._selected.constructor == Page && navigationDiagram._selected._type=='subsection')))
    {
        if (xmlhttpString == viewString)
        {
            ViewXMLHttp.open("POST", 'FormMaker?action=process&operation1=load_node_list&layer_id=' + xmlhttpString, true);
            ViewXMLHttp.onreadystatechange = showList;
            ViewXMLHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            ViewXMLHttp.send(createProjectInfoBodyForAJAX());
        }
        else if (xmlhttpString == controllerString)
        {
            if(status == 'new') _newHandler = 'new';
            else _newHandler = '';
            ControllerXMLHttp.open("POST", 'FormMaker?action=process&operation1=load_node_list&layer_id=' + xmlhttpString, true);
            ControllerXMLHttp.onreadystatechange = showList;
            ControllerXMLHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            ControllerXMLHttp.send(createProjectInfoBodyForAJAX());
        }
        else if (xmlhttpString == modelString)
        {
            ModelXMLHttp.open("POST", 'FormMaker?action=process&operation1=load_node_list&layer_id=' + xmlhttpString, true);
            ModelXMLHttp.onreadystatechange = showList;
            ModelXMLHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            ModelXMLHttp.send(createProjectInfoBodyForAJAX());
        }
        else if (xmlhttpString == selectedNodeString)
        {
            if(navigationDiagram._selected.constructor == Page && navigationDiagram._selected._type == "subsection" && hasSubsectionHandler == 'true')
            {
                SelectedNodeXMLHttp.open("POST", 'FormMaker?action=process&operation1=load_node_list&node_id=' + navigationDiagram._handlers[navigationDiagram._selected._id]._nodeID + '&pattern_id=' + navigationDiagram._handlers[navigationDiagram._selected._id]._pattern + '&direction=outgoing', true);
            }
            else
            {
                SelectedNodeXMLHttp.open("POST", 'FormMaker?action=process&operation1=load_node_list&node_id=' + navigationDiagram._selected._nodeID + '&pattern_id=' + navigationDiagram._selected._pattern + '&direction=outgoing', true);
            }
            SelectedNodeXMLHttp.onreadystatechange = showList;
            SelectedNodeXMLHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            SelectedNodeXMLHttp.send(createProjectInfoBodyForAJAX());
        }
    }
}

/**
 * Convenience method... Finalises form ID details and submits page
 */
function formIDSets()
{
    if ((navigationDiagram.getSelected() == null) || (navigationDiagram.getSelected().constructor != Page))
    {
        if ((navigationDiagram.getSelected() != null) && (navigationDiagram.getSelected().constructor == Link))
        {
            if (navigationDiagram.getSelected()._source.constructor == Page)
            {
                var form_id;
                for(form in navigationDiagram.getSelected()._source._forms)
                {
                    form_id = navigationDiagram.getSelected()._source._forms[form]._id;
                }
                formIDSetsSubmit(navigationDiagram.getSelected()._source._name, form_id);
            }
            else
                alert('Please select a page on the diagram to use this function.');
        }
        else
            alert('Please select a page on the diagram to use this function.');
    }
    else
    {
        var form_id;
        for(form in navigationDiagram.getSelected()._forms)
        {
            form_id = navigationDiagram.getSelected()._forms[form]._id;
        }
        formIDSetsSubmit(navigationDiagram.getSelected()._name, form_id);
    }
}

function formIDSetsSubmit(page_name, form_id)
{
    document.result_form.page_source_type.value = navigationDiagram._forms[form_id]._sourceType;
    document.result_form.schema_filename.value = navigationDiagram._forms[form_id]._sourceFile;
    document.result_form.schema_root_element.value = navigationDiagram._forms[form_id]._rootElement;

    //request operation can never be set for a page anymore, as this is now set on the actions
    document.result_form.request_operation.value = '';
    document.result_form.response_operation.value = navigationDiagram._forms[form_id]._responseOperation;

    navigationDiagram._forms[form_id]._structureGenned = 'true';

    document.result_form.form_id.value = form_id;

    //set the gen control attribute for the current page
    document.result_form.gen_control.value = page_name;
    document.result_form.gen_control_type.value = 'Page';

    //set the preview URL value based on the selected form
    if (typeof(navigationDiagram._preview._url) != 'undefined')
    {
        document.result_form.preview_url.value = navigationDiagram._preview._url + "/"
        + 'Page_Preview_' + page_name + '.html';
    }
    else
    {
        document.result_form.preview_url.value = "";
    }

    //check for any links for which an action name has not been entered.
    //These can cause problems with the bindings page (and other functions), so we make sure
    //they have all been entered before continuing.
    for (link in navigationDiagram._links)
    {
        var lk = navigationDiagram._links[link];
        if (lk._action == '')
        {
            alert('Please ensure that the action name for every link in the application has been correctly entered before continuing.');
            //+'\n\nThe link from the ' + lk._source._name + ' ' + lk._source.constructor + ' to the ' + lk._target._name + ' ' + lk._target.Constructor + ' has not been named.');
            return;
        }
    }

    document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();
    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);
    document.result_form.submit();
}


function genControl()
{
    if (navigationDiagram.getSelected() != null)
    {
        if (navigationDiagram.getSelected().constructor == Link)
        {
            document.result_form.gen_control_type.value = 'Link';
            document.result_form.gen_control.value = navigationDiagram.getSelected()._id;
        }
        else
        {
            if (navigationDiagram.getSelected().constructor == Handler)
            {
                document.result_form.gen_control_type.value = 'Handler';
            }
            else
                document.result_form.gen_control_type.value = 'Page';
            document.result_form.gen_control.value = navigationDiagram.getSelected()._name;
        }
    }
    else
        document.result_form.gen_control.value = '';
}

function toggleDefaults()
{
    if (defaultsMode == 'visible')
    {
        hideComponent(document.getElementById("project_defaults_table"));
        defaultsMode = 'hidden';
        setTextContent(document.getElementById("show_app_defaults"), 'Show Advanced Options');
    }
    else
    {
        showComponent(document.getElementById("project_defaults_table"));
        defaultsMode = 'visible';
        setTextContent(document.getElementById("show_app_defaults"), 'Hide Advanced Options');
    }
}

function setAccessibility(acc)
{
    for (page in navigationDiagram._pages)
    {
        if (navigationDiagram._pages[page]._accessibilityLevel == navigationDiagram._defaults._accessibilityLevel)
        {
            navigationDiagram._pages[page]._accessibilityLevel = acc.value;
            if ((navigationDiagram.getSelected() != null) && (navigationDiagram.getSelected().constructor == Page) && (navigationDiagram.getSelected()._id == navigationDiagram._pages[page]._id))
                navigationDiagram._pages[page].displayContextPanel();
        }
    }
    navigationDiagram._defaults._accessibilityLevel = acc.value;
    navigationDiagram.setChanged();
}

/**
 * Checks that the given node still exists in the list of ndoes of the given type.
 * @param node The full 4 part node id to check
 * @param listType the type of nodes to check - 'Controller' or 'View'
 */
function nodeChecker(node, listType)
{
    var nodeCheck = 'false'
    var semiNode = '';
    var fullNode = '';
    var nodeList = null;
    if (listType == 'Controller')
    {
        if (controllerNodeList == null)
            populateXMLNodeList(controllerString);
        if (controllerNodeList != null)
            nodeList = controllerNodeList.selectNodes("/xfact:patterns/xfact:pattern");
    }
    else
    {
        if (viewNodeList == null)
            populateXMLNodeList(viewString);
        if (viewNodeList != null)
            nodeList = viewNodeList.selectNodes("/xfact:patterns/xfact:pattern");
    }

    if (nodeList != null)
    {
        for (var i=0; i < nodeList.length; i++)
        {
            semiNode = nodeList.item(i);
            fullNode = navigationDiagram._projectInfo + '-' + semiNode.getAttribute("pattern_id") + '-' + getElementText(semiNode.selectSingleNode("xfact:nodes/xfact:node/@node_id"));
            if (fullNode == node)
            {
                nodeCheck = 'true';
                break;
            }
        }
    }

    return nodeCheck;
}

function displayEmptyPanel(message){
    var xml = "<message>" + message + "</message>";
    emptyTransform.transform(getXMLDocument(xml));
}

function displayMessagePanel(message){
    var xml = "<message>" + message + "</message>";
    messageTransform.transform(getXMLDocument(xml));
}

/*******************************

  * Main operation functions of the Form Map screen

******************************/




/**
 * Resets the list of operations that will be performed on the next submit.
 * This should always be called before you start defining the required operations.
 */
function resetOperationsList()
{

    var i = 1;
    var opField = document.result_form.operation1;

    while (opField != null)
    {
        opField.value = '';
        i++;
        opField = eval('document.result_form.operation' + i);
    }
}

/**
 * Adds the given operation to the list of operations to be performed when the page is submitted.
 */
function addOperation(opName)
{
    var i = 1;
    var opField = document.result_form.operation1;

    while (opField != null && opField.value != '')
    {
        i++;
        opField = eval('document.result_form.operation' + i);
    }

    if (opField == null)
    {
        if (dojo.isIE < 8)
            opField = document.createElement('<input name="operation'+i+'"/>');
        else
            opField = document.createElement('input');
        opField.type = 'hidden';
        opField.name = 'operation' + i;
        document.result_form.appendChild(opField);
    }

    opField.value = opName;
}

/**
 * checks if any other server side process are needed to save the chanegs made on screen.
 * and if so adds the appropriate operations etc.
 * Currently only checks for action processing.
 */
function checkServerProcessRequired()
{
    if (navigationDiagram._actionChanges.length > 0)
    {
        addOperation('process_action_changes');
        var field = document.result_form.xml_actionChanges;
        if (field == null)
        {
            field = document.createElement('input');
            field.type = 'hidden';
            field.name = 'xml_actionChanges';
            document.result_form.appendChild(field);
        }
        field.value = navigationDiagram.getActionChangesXML();
    }
}


 /**
 * Save application XML
 */
function saveApplicationXML()
{
    resetOperationsList()
    document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_map');
    document.result_form.presentation_xsl.value = 'form_map_html.xsl';
    //set the gen control attribute for the current page
    genControl();
    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);
    document.result_form.submit();
}

function discardApplicationXML()
{
    if (confirm('This will reset all details to their initial values upon loading the page.\nDo you wish to continue?'))
    {
        resetOperationsList()
        addOperation('load_form_map');
        document.result_form.presentation_xsl.value = 'form_map_html.xsl';
        genControl();
        //stop the win manager prompting about losing changes
        WinManager.setGenuineCloseFlag(false);
        document.result_form.submit();
    }
}

/**
 * Previews the selected page
 */
function previewPage()
{
    if ((navigationDiagram.getSelected() == null) || (navigationDiagram.getSelected().constructor != Page))
        alert('Please select a page on the diagram to use this function.');
    else
    {
        if (typeof(navigationDiagram._preview._url) != 'undefined')// && (navigationDiagram._forms[form_id]._dataToForm._value != ''))
        {
            var pageName = navigationDiagram._selected._name;
            var previewUrl = navigationDiagram._preview._url + '/Page_Preview_' + pageName + '.html';

            var targetUrl = 'page_preview.html?page=' + pageName + '&url=' + previewUrl;
            WinManager.openNewChildWindowStart(targetUrl, "Page Preview", fm_project, pageName);
        }
        else
            alert('Please select a Preview Location before generating and previewing the page.');
    }
}

/**
 * Submits the form and goes to the form details page
 */
function gotoDetailsPage()
{
    document.result_form.presentation_xsl.value = 'field_details_html.xsl';
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_form_map');

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    genControl()
    formIDSets()
}

/**
 * Submits the form and goes to the form details page
 */
function gotoStructuresPage()
{
    document.result_form.presentation_xsl.value = 'form_structure_html.xsl';
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_bindings_file');
    addOperation('load_form_map');

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    genControl()
    formIDSets()
}

/**
 * Submits the form and goes to the form details page
 */
function gotoBindingsPage()
{
    document.result_form.presentation_xsl.value = 'xpath_bindings_html.xsl';

    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_bindings_file');
    addOperation('load_form_map');

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    // check if a link is selected. if it is set the correct attributes
    if ((navigationDiagram.getSelected() != null) && (navigationDiagram.getSelected().constructor == Link))
    {
        document.result_form.currentTab.value = 'page-to-handler';
        document.result_form.currentAction.value = navigationDiagram.getSelected()._action;
    }

    genControl()
    formIDSets()
}


/**
 * Submits the page to go back to the server to generate example documents for the current section
 */
function generateExampleDocuments()
{
    var selectedObj = objectType(navigationDiagram.getSelected());

    if (selectedObj == 'Page')
    {
        var form_id;
        for(form in navigationDiagram.getSelected()._forms)
        {
            form_id = navigationDiagram.getSelected()._forms[form]._id;
        }

        document.result_form.form_id.value = form_id;
        document.result_form.page_source_type.value = navigationDiagram._forms[form_id]._sourceType;
        document.result_form.schema_filename.value = navigationDiagram._forms[form_id]._sourceFile;
        document.result_form.schema_root_element.value = navigationDiagram._forms[form_id]._rootElement;
        document.result_form.response_operation.value = navigationDiagram._forms[form_id]._responseOperation;
        document.result_form.request_operation.value = "";
    }
    else if (selectedObj == 'Link')
    {
        if(document.getElementById('action_name').value == "")
        {
            alert("Please enter an Action Name");
            return;
        }
        document.result_form.page_source_type.value = navigationDiagram.getSelected()._sourceType;
        document.result_form.schema_filename.value = navigationDiagram.getSelected()._sourceFile;
        document.result_form.schema_root_element.value = navigationDiagram.getSelected()._rootElement;
        document.result_form.request_operation.value = navigationDiagram.getSelected()._requestOperation;
        document.result_form.response_operation.value = "";
    }

    if ((document.result_form.schema_filename != null) && (document.result_form.schema_filename.value == ""))
    {
        alert('Please select a data structure file for this page/action before using this function.');
        return;
    }


    resetOperationsList()
    addOperation('save_form_map');
    addOperation('generate_example_documents');
    checkServerProcessRequired();
    addOperation('load_form_map');

    document.result_form.presentation_xsl.value = 'form_map_html.xsl';

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    genControl();


    document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();
    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);
    document.result_form.submit();
}


/************************************

    Functions for showing various screen details

*************************************/

/** Gets and displays a list of controller/view nodes after HTTP call is ready - XDE integration functionality */
function showList()
{
    if ((navigationDiagram._selected != null) && (typeof(navigationDiagram._selected) != 'undefined'))
    {

        if (navigationDiagram._selected.constructor == Link)
        {
            if (ViewXMLHttp.readyState==4)
            {
                viewNodeList = getXMLDocument(ViewXMLHttp.responseText);
                Sarissa.setXpathNamespaces(viewNodeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                ViewXMLHttp.abort();
            }

            if (ControllerXMLHttp.readyState==4)
            {
                controllerNodeList = getXMLDocument(ControllerXMLHttp.responseText);
                Sarissa.setXpathNamespaces(controllerNodeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                ControllerXMLHttp.abort();
            }
            if ((viewNodeList != null) & (controllerNodeList != null))
            {
                if ((viewNodeList.selectSingleNode("/fm:error") != null) | (controllerNodeList.selectSingleNode("/fm:error") != null))
                {
                    alert('There was a problem with the requested operation. Please notify your system administrator.');
                }

                navigationDiagram._selected.displayContextPanel();
            }
        }
        else
        {
            if ((ControllerXMLHttp.readyState==4) && (serviceLinked == true))
            {
                controllerNodeList = getXMLDocument(ControllerXMLHttp.responseText);
                Sarissa.setXpathNamespaces(controllerNodeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                ControllerXMLHttp.abort();
            }

            if (ModelXMLHttp.readyState==4)
            {
                modelNodeList = getXMLDocument(ModelXMLHttp.responseText);
                Sarissa.setXpathNamespaces(modelNodeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                ModelXMLHttp.abort();
            }
            if (SelectedNodeXMLHttp.readyState==4)
            {
                selectedNode_nodeList = getXMLDocument(SelectedNodeXMLHttp.responseText);
                Sarissa.setXpathNamespaces(selectedNode_nodeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                serviceLinked = true;
                SelectedNodeXMLHttp.abort();
            }
            if ((controllerNodeList != null) & (modelNodeList != null) & ((navigationDiagram._handlers[navigationDiagram._selected._id]._node == "") | ((navigationDiagram._handlers[navigationDiagram._selected._id]._node != "") & (selectedNode_nodeList != null))))
            {

                if ((modelNodeList.selectSingleNode("/fm:error") != null) | (controllerNodeList.selectSingleNode("/fm:error") != null) | ((navigationDiagram._handlers[navigationDiagram._selected._id]._node != "") && (selectedNode_nodeList.selectSingleNode("/fm:error") != null)))
                {
                    alert('There was a problem with the requested operation. Please notify your system administrator.');
                }

                if(selectedNode_nodeList != null)
                {
                    if(navigationDiagram._selected.constructor == Handler)
                    {
                        document.getElementById('message_context_panel').style.display = "none";
                        document.getElementById('context_panel').style.display = "none";
                        document.getElementById('xtra_context_panel').style.display = "none";
                        document.getElementById('handler_container_context_panel').style.display = "block";
                    }
                    if(navigationDiagram._selected.constructor == Page && navigationDiagram._selected._type == 'subsection')
                    {
                        document.getElementById('message_context_panel').style.display = "none";
                        document.getElementById('handler_container_context_panel').style.display = "block";
                    }
                }
                if(_newHandler == 'new')
                {
                    navigationDiagram._selected.displayContextPanel('new');
                    _newHandler = ''
                }
                else
                    navigationDiagram._selected.displayContextPanel();
            }
        }
    }
}

/** Similar to showList, but checks for success message of node operation. If successful, calls populateXMLNodeList*/
function showOutcome()
{
    if (xmlhttp.readyState==4)
    {
        var outcomeList = getXMLDocument(xmlhttp.responseText);
        if ((navigationDiagram._selected != null) && (typeof(outcomeList) != undefined))
        {
            Sarissa.setXpathNamespaces(outcomeList, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
            if (outcomeList.selectSingleNode("/fm:error") != null)
            {
                alert('There was a problem with the requested operation. Please notify your system administrator.');
            }
            else
            {
                //displayEmptyPanel('Operation successful. Refreshing window...');
                if (serviceLinked == false)
                {
                    xmlhttp = SelectedNodeXMLHttp;
                }
                if (xmlhttp == ViewXMLHttp)
                {
                    viewNodeList = null;
                    populateXMLNodeList(viewString);
                }
                else
                {
                    if (xmlhttp == ControllerXMLHttp)
                    {
                        //now that a new controller has been created we need to refresh both the
                        //controller list, and the list of available services.
                        //The second is needed because the current handler will be set to use the new controller
                        controllerNodeList = null;
                        populateXMLNodeList(controllerString);
                        selectedNode_nodeList = null;
                        populateXMLNodeList(selectedNodeString);
                    }
                    else if (xmlhttp == ModelXMLHttp)
                    {
                        modelNodeList = null;
                        populateXMLNodeList(modelString);
                    }
                    else if (xmlhttp == SelectedNodeXMLHttp)
                    {
                        selectedNode_nodeList = null;
                        populateXMLNodeList(selectedNodeString,'true');
                    }
                    else
                        navigationDiagram._selected.displayContextPanel();

                    //select the newly added subsection if it has just been created.
                    if(navigationDiagram._selected.constructor == Handler && navigationDiagram._selected._type != null
                        && navigationDiagram._selected._type == "subsection")
                    {
                        navigationDiagram.setSelected(navigationDiagram._pages[navigationDiagram._selected._id]);
                    }
                }
            }
        }
        else //ie when deletePageXSL has run
        {
        }
    }
}

/** Gets the information related to the selected Action on a Controller window */
function showServices(handlerID, nodeName)
{
    selectedNode_nodeList = null;

    navigationDiagram._handlers[handlerID].getNodeInfo(nodeName);

    if (navigationDiagram._handlers[handlerID]._node != "")
        populateXMLNodeList(selectedNodeString);

    if (document.result_form.controllers_list.value == "")
        navigationDiagram._handlers[handlerID].hideServices();
}

/** Gets the extra information on the selected View Node on an ActualLink window */
function showNode(linkID, nodeName)
{
    navigationDiagram._links[linkID].getNodeInfo(nodeName);
    navigationDiagram._links[linkID].displayContextPanel();
}

/* Displays the extra information on the given form */
function showForm(formName)
{
    if(typeof(formName != 'undefined') && formName != null && formName != '')
    {
        var form = navigationDiagram._forms[formName];

        form.displayContextPanel();
    }
    else
    {
        document.getElementById('xtra_context_panel').innerHTML = "";
        document.getElementById('handler_container_context_panel').innerHTML = "";
    }
}

/*****************************

    Form handling functions

******************************/

/* Adds a form to the given page */
function addForm(pageID, formID)
{
    if (formID == "")
        alert("Please select a section to use this function");
    else
    {
        var page = navigationDiagram._pages[pageID];
        var form = navigationDiagram._forms[formID];

        if (getPropertyCount(page._forms) > 0)
        {
            alert('Each page can currently only contain one section.\nPlease remove the existing section before adding the new one.');
        }
        else
        {
            page._forms[formID] = form;
            page.setChanged();
            page.displayContextPanel();
        }
    }
}

/* Removes a form from the given page */
function removeForm(pageID, formID)
{
    var page = navigationDiagram._pages[pageID];
    delete page._forms[formID];
    page.setChanged();
    page.displayContextPanel();
}

/**
 * Deletes a form from the system, and removes it from the dropdown list
 */
function deleteForm(pageID, formID)
{
    if (formID == "")
        alert("Please select a section to use this function");
    else
    {
        //delete the form object
        delete navigationDiagram._forms[formID];

        //remove it from any pages still using it
        for(page in navigationDiagram._pages)
        {
            delete navigationDiagram._pages[page]._forms[formID];
        }

        //reshow the current page context panel
        var page = navigationDiagram._pages[pageID];
        page.setChanged();

    }
}

/**
 * Create a new form
 */
function newForm(suggestedFormName)
{
    var inputchek = false;
    var formName = suggestedFormName;

    do
    {
        formName = prompt("Please give a unique name.", formName);
        inputchek = ((checkInvalidNames(formName)) || (checkInvalidChars(formName)) || (checkPropertyNameExists(formName, navigationDiagram._forms)));
        if ((formName != null) & (inputchek == false))
        {
            if (inputchek == false)
            {
                var newForm = new Form(formName, navigationDiagram);

                navigationDiagram._forms[formName] = newForm;
                navigationDiagram._selected._gennedSection = formName;
                navigationDiagram._selected.displayContextPanel();
                navigationDiagram.setChanged();
            }
        }
    }
    while ((inputchek == true) & (formName != null));
}


/***************************

    Functions for generating new instances of elements on the Form Map screen

****************************/

/**
 * Calls to the server to generate output files for evey page in the application
 */
function genAll(operation)
{
    document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();

    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();

    if (navigationDiagram._displayType == 'formmaker')
    {
        addOperation('generate_output');
        addOperation('load_form_map');
    }
    else
    {
        if (operation == 'pages')
        {
            addOperation('generate_output');
            addOperation('load_form_map');
        }
        else if (operation == 'handlers')
        {
            addOperation('generate_handlers');
            addOperation('load_form_map');
        }
        else
        {
            addOperation('generate_output');
            addOperation('generate_handlers');
            addOperation('load_form_map');
        }
    }


    document.result_form.presentation_xsl.value = 'form_map_html.xsl';
    document.result_form.gen_control.value = 'all';
    showStatusMessage("Generation in process...");
    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);
    document.result_form.submit();
}

/** this ios the function called from teh top level app menu to generate the application,
 * so for this screen we just forward on to the existing gen functionality. */
function generateApp()
{
    genAll('all');
}

function genPattern(layer, handlerName, createPrompt)
{
    var inputchek = false;
    var targetDocument = handlerName;
    var nodeString = "";

    do
    {
        if(createPrompt != null && createPrompt == 'false')
        {
            targetDocument = handlerName;
        }
        else
        {
            targetDocument = prompt("Please give a unique name.", targetDocument);
        }

        inputchek = (checkInvalidNames(targetDocument) || checkInvalidChars(targetDocument));
        if ((targetDocument != null) && (inputchek == false))
        {
            var documentCheck;
            if (layer == 'L1')
                documentCheck = viewNodeList.selectNodes("//xfact:pattern[./@pattern_id = '" + targetDocument + "']");
            else if (layer == 'L2')
                documentCheck = controllerNodeList.selectNodes("//xfact:pattern[./@pattern_id = '" + targetDocument + "']");
            else
                documentCheck = modelNodeList.selectNodes("//xfact:pattern[./@pattern_id = '" + targetDocument + "']");

            if(documentCheck.length == 0)
            {
                if (targetDocument.indexOf('.xml') == -1)
                targetDocument = targetDocument + '.xml';

                var sourceXstore;
                var sourceCollection;
                var sourceDocument;
                if (layer == controllerString)
                {
                    sourceXstore = sourceControllerXstore
                    sourceCollection = sourceControllerCollection
                    sourceDocument = sourceControllerDocument;
                    targetXstore = targetControllerXstore;
                    targetCollection = targetControllerCollection;
                    xmlhttp = ControllerXMLHttp;
                }
                else
                {
                    sourceXstore = sourceViewXstore
                    sourceCollection = sourceViewCollection
                    sourceDocument = sourceViewDocument;
                    targetXstore = targetViewXstore;
                    targetCollection = targetViewCollection;
                    xmlhttp = ViewXMLHttp;
                }

                xmlhttp.open("POST", 'FormMaker?action=process&operation1=clone_pattern&source_xstore=' + sourceXstore + '&source_collection=' + sourceCollection + '&source_document=' + sourceDocument + '&target_xstore=' + targetXstore + '&target_collection=' + targetCollection + '&target_document=' + targetDocument +  '&layer=' + layer, true);

                xmlhttp.onreadystatechange = showOutcome;
                xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
                xmlhttp.send(createProjectInfoBodyForAJAX());

                //set the handler to use this new controller
                var newPattern;
                if (targetDocument.indexOf('.xml') != -1)
                    newPattern = targetDocument.slice(0, targetDocument.lastIndexOf("."));
                if (navigationDiagram._selected.constructor == Handler)
                {
                    navigationDiagram._selected.getNodeInfo(newPattern + '-Controller');
                }
            }
            else
            {
                if(createPrompt != null && createPrompt == 'false')
                {
                    alert('Could not automatically create controller due to naming conflict. Please create or assign one using the advanced options.');
                    inputchek = false;
                    document.getElementById('message_context_panel').style.display = "none";
                    document.getElementById('handler_container_context_panel').style.display = "block";
                    if(navigationDiagram._selected._type=='normal')
                    {
                        document.getElementById('context_panel').style.display = "none";
                        document.getElementById('xtra_context_panel').style.display = "none";
                    }
                    else if(navigationDiagram._selected._type = 'subsection')
                    {
                        document.getElementById('context_panel').style.display = "block";
                        document.getElementById('xtra_context_panel').style.display = "block";
                        var pageID = navigationDiagram._selected._id;
                        navigationDiagram.setSelected(undefined);
                        navigationDiagram.selectedNode_nodeList = null;
                        navigationDiagram.setSelected(navigationDiagram._pages[pageID]);
                        document.getElementById('message_context_panel').style.display = "none";
                        document.getElementById('handler_container_context_panel').style.display = "block";
                    }
                }
                else
                {
                    alert('Name already in use. Please specify another unique name.');
                    inputchek = true;
                }
            }
        }
    }
    while ((inputchek == true) & (targetDocument != null));
}

/**
 * Calls to the server to generate the output for the page with id page_id
 * @param page_id The id of the page to generate
 */
function generatePage()
{
    if ((navigationDiagram.getSelected() == null) || (navigationDiagram.getSelected().constructor != Page))
        alert('Please select a page on the diagram to use this function.');

    else
    {
        var page_id = navigationDiagram.getSelected()._id;
        document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();

        resetOperationsList()
        addOperation('save_form_map');
        checkServerProcessRequired();
        addOperation('generate_output');
        addOperation('load_form_map');

        document.result_form.presentation_xsl.value = 'form_map_html.xsl';

        genControl();

        var form_id;
        for(form in navigationDiagram.getSelected()._forms)
        {
            form_id = form;
            break;
        }
        document.result_form.schema_filename.value = navigationDiagram._forms[form_id]._sourceFile;
        document.result_form.schema_root_element.value = navigationDiagram._forms[form_id]._rootElement;
        document.result_form.form_id.value = form_id;
        showStatusMessage("Generation in process...");
        //stop the win manager prompting about losing changes
        WinManager.setGenuineCloseFlag(false);
        document.result_form.submit();
    }
}

function genPageAndLogicsheet()
{
    if ((navigationDiagram.getSelected() == null) || (navigationDiagram.getSelected().constructor != Page))
        alert('Please select a page on the diagram to use this function.');
    else
    {
        var page_id = navigationDiagram.getSelected()._id;
        document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();

        resetOperationsList()
        addOperation('save_form_map');
        checkServerProcessRequired();
        addOperation('gen_logicsheet');
        addOperation('generate_output');
        addOperation('load_form_map');

        document.result_form.presentation_xsl.value = 'form_map_html.xsl';
        genControl();
        var form_id;
        for(form in navigationDiagram.getSelected()._forms)
        {
            form_id = form;
            break;
        }
        document.result_form.schema_filename.value = navigationDiagram._forms[form_id]._sourceFile;
        document.result_form.schema_root_element.value = navigationDiagram._forms[form_id]._rootElement;
        document.result_form.form_id.value = form_id;
        //stop the win manager prompting about losing changes
        WinManager.setGenuineCloseFlag(false);
        document.result_form.submit();
    }
}

function genLogicsheet()
{
    document.result_form.xml_formMap.value = navigationDiagram.getApplicationXML();
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('gen_logicsheet');
    addOperation('load_form_map');
    document.result_form.presentation_xsl.value = 'form_map_html.xsl';
    genControl();
    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);
    document.result_form.submit();
}

function deploySelectedPage()
{
    if ((navigationDiagram.getSelected() == null) || (navigationDiagram.getSelected().constructor != Page))
    {
        alert('Please select a page on the diagram to use this function.');
        return;
    }
    genControl();
    deployHandler('page');
}

function genServiceLink(serviceNode)
{
    if (serviceNode == "")
        alert("Please select a service to use this function");
    else
    {
        serviceLinked = false;
        var nodeArray = serviceNode.split('-');
        servicePattern = nodeArray[0];
        serviceNodeID = nodeArray[1];
        var selectedNodeID;
        var selectedPattern;

        if(navigationDiagram._selected.constructor == Page && navigationDiagram._selected._type=="subsection")
        {
            selectedPattern = navigationDiagram._handlers[navigationDiagram._selected._id]._pattern;
            selectedNodeID = navigationDiagram._handlers[navigationDiagram._selected._id]._nodeID;
        }
        else
        {
            selectedNodeID = navigationDiagram._selected._nodeID;
            selectedPattern = navigationDiagram._selected._pattern;
        }

        xmlhttp = SelectedNodeXMLHttp;
        xmlhttp.open("POST", 'FormMaker?action=process&operation1=add_splice&source_pattern_id='+ selectedPattern//navigationDiagram._selected._pattern
            + '&source_node_id='+ selectedNodeID /*navigationDiagram._selected._nodeID*/ + '&target_pattern_id='+ servicePattern
            + '&target_node_id='+ serviceNodeID + '&splice_action=invoke&replace_existing=true', true);
        xmlhttp.onreadystatechange = showOutcome;
        xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        xmlhttp.send(createProjectInfoBodyForAJAX());
    }
}

function removeServiceLink(controllerNode, serviceNode)
{
    if (serviceNode == "")
        alert("Please select a service to use this function");
    else
    {
        serviceLinked = false;
        var nodeArray = serviceNode.split('-');
        servicePattern = nodeArray[0];
        serviceNodeID = nodeArray[1];

        xmlhttp = SelectedNodeXMLHttp;
        xmlhttp.open("POST", 'FormMaker?action=process&operation1=remove_service_link'
           + '&rule_xstore='+ 'blueprints' + '&rule_collection=' + 'mvcuser/mvc/Example/logicsheet_pool'
           + '&rule_document=' + navigationDiagram._handlers[navigationDiagram._selected._id]._pattern + '_' + navigationDiagram._handlers[navigationDiagram._selected._id]._nodeID + '_rules.xml'
           + '&rule_pattern_id='+ navigationDiagram._handlers[navigationDiagram._selected._id]._pattern + '&rule_node_id=' + navigationDiagram._handlers[navigationDiagram._selected._id]._nodeID
           + '&splice_source_pattern_id='+ navigationDiagram._handlers[navigationDiagram._selected._id]._pattern + '&splice_source_node_id=' + navigationDiagram._handlers[navigationDiagram._selected._id]._nodeID
           + '&splice_target_pattern_id='+ servicePattern + '&splice_target_node_id=' + serviceNodeID
           + '&rule_id=call_' + serviceNode + '_from_' + controllerNode + '_rule', true);
        xmlhttp.onreadystatechange = showOutcome;
        xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        xmlhttp.send(createProjectInfoBodyForAJAX());
    }
}

/**
 * Encodes the project information XML fragment (from the hidden field)
 * for use in an AJAX post call, and returns the created string.
 */
function createProjectInfoBodyForAJAX()
{
    var bodyString = 'xml_project_info=';
    bodyString += URLencode(document.result_form.xml_project_info.value);
    return bodyString;
}


function URLencode(sStr)
{
    return escape(sStr).
        replace(/\+/g, '%2B').
            replace(/\"/g,'%22').
               replace(/\'/g, '%27');
}
/*******************************

    Functions to handle document information for some of the data fields within the Form Details window

********************************/


/** Global variable to simplify communication with xstore call back function */
var documentNameField;

/**
 * Makes a call to XStoreViewer to select a skin file
 * @param type The type of skin to adjust, 'application' or 'page'
 */
function selectSkinFile(type)
{
    if (type == 'application')
        var currentValue = navigationDiagram._defaults._skin;
    else
        var currentValue = navigationDiagram._selected._skin;

    document.xstoreViewerCaller.xml_permissions.value =
        "<xstores filter='on' xmlns='http://www.hyfinity.com/xengine'>" +
            "<xstore name='" + outputXStore + "'>" +
                "<collections>" +
                    "<collection name='" + outputCollection + "'/>" +
                "</collections>" +
            "</xstore>" +
        "</xstores>";
    document.xstoreViewerCaller.chosen_xstore_name.value = outputXStore;
    document.xstoreViewerCaller.chosen_collection_name.value = outputCollection;
    document.xstoreViewerCaller.chosen_document_name.value = currentValue;
    window.open('about:blank','xstoreViewerWindow','width=1000, height=600, toolbar=yes, menubar=no, location=no, directories=no, scrollbars=yes, resizable=yes, status=yes');

    document.xstoreViewerCaller.selection_mode.value = 'id';

    documentNameField = type;
    window.xstoreViewer = setSkinXstoreViewerCallback;
    document.xstoreViewerCaller.submit();
}

function setSkinXstoreViewerCallback(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId)
{
    if (documentNameField == 'application')
    {
        navigationDiagram._defaults._skin = documentName;
        setTextContent(document.getElementById('page_skin'), documentName);
        navigationDiagram.setChanged();
    }
    else
    {
        navigationDiagram._selected._skin = documentName;
        navigationDiagram._selected.setChanged();
        navigationDiagram._selected.displayContextPanel();
    }
}

/**
 * function to call XStoreViewer when selecting a data structure file. (schema or WSDL)
 */
function selectSectionStructureDocument(form, xstore, collection)
{
    document.xstoreViewerCaller.xml_permissions.value =
        "<xstores filter='on' xmlns='http://www.hyfinity.com/xengine'>" +
            "<xstore name='" + xstore + "'>" +
                "<collections>" +
                    "<collection name='" + collection + "'/>" +
                "</collections>" +
            "</xstore>" +
        "</xstores>";
    document.xstoreViewerCaller.chosen_xstore_name.value = xstore;
    document.xstoreViewerCaller.chosen_collection_name.value = collection;
    document.xstoreViewerCaller.chosen_document_name.value = '';
    window.open('about:blank','xstoreViewerWindow','width=1000, height=600, toolbar=yes, menubar=no, location=no, directories=no, scrollbars=yes, resizable=yes, status=yes');

    document.xstoreViewerCaller.selection_mode.value = 'content';

    documentNameField = form;
    window.xstoreViewer = contentXstoreViewer;
    document.xstoreViewerCaller.submit();
}

/**
 * XStoreViewer callback function when selecting a data structure file.
 * This returns the file content as well as the ID information.
 */
function contentXstoreViewer(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content)
{
    if(navigationDiagram._selected.constructor == Page)
    {
        navigationDiagram._forms[documentNameField].sourceFileSelected(documentName, content);
    }
    else if(navigationDiagram._selected.constructor == Link)
    {
        navigationDiagram._links[documentNameField].sourceFileSelected(documentName, content);
    }
}

/**
 * Makes a call to xstore to select a document, collection and xstore
 * @param xstore Teh name of the xstore the document should be selected from.
 * @param collections An array of Strings detailling all teh collections that can be used.
 */
function selectInstance(dataType, formJavascriptIdentifier, field, xstore, collections)
{
    //check if the xstore and collections arrays have been specified
    if ((typeof(xstore) == 'undefined') || (typeof(collections) == 'undefined') || (collections == null) || (collections.length == 0))
    {
        //not specified, so dont restrict display
        document.xstoreViewerCaller.chosen_xstore_name.value = '';
        document.xstoreViewerCaller.chosen_collection_name.value = '';
        document.xstoreViewerCaller.chosen_document_name.value = '';
        //disable the chosen colelction values so that they wont be submitted
        document.xstoreViewerCaller.chosen_xstore_name.disabled = true;
        document.xstoreViewerCaller.chosen_collection_name.disabled = true;
        //turn the display filtering off
        document.xstoreViewerCaller.xml_permissions.value = "<xstores filter='off' xmlns='http://www.hyfinity.com/xengine'>";
    }
    else
    {
        document.xstoreViewerCaller.xml_permissions.value = "<xstores filter='on' xmlns='http://www.hyfinity.com/xengine'>";
        document.xstoreViewerCaller.chosen_xstore_name.disabled = false;
        document.xstoreViewerCaller.chosen_collection_name.disabled = false;
        document.xstoreViewerCaller.chosen_document_name.value = '';
        document.xstoreViewerCaller.chosen_xstore_name.value = xstore;
        document.xstoreViewerCaller.chosen_collection_name.value = collections[0];
     }

     document.xstoreViewerCaller.xml_permissions.value += "<xstore name='" + xstore + "'>" +
                                                            "<collections>";

    for(i in collections)
    {
        document.xstoreViewerCaller.xml_permissions.value += "<collection name='" + collections[i] + "'/>";
    }

    document.xstoreViewerCaller.xml_permissions.value +=  "</collections>" +
        "</xstore>" +
        "</xstores>";

    document.xstoreViewerCaller.selection_mode.value = 'id';

    window.open('about:blank','xstoreViewerWindow','width=1000, height=600, toolbar=yes, menubar=no, location=no, directories=no, scrollbars=yes, resizable=yes, status=yes');

    documentNameField = field;
    xtraFormJavascriptIdentifier = formJavascriptIdentifier;
    xtraDataType = dataType;
    window.xstoreViewer = xtraXStoreViewer;
    document.xstoreViewerCaller.submit();
}

/** Global variable to simplify call to xstore for instances */
var xtraFormJavascriptIdentifier;
var xtraDataType;

/**
 * Simple function to provide xstore call back functionality for instances
 */
function xtraXStoreViewer(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId)
{
    documentNameField.value = documentName;
    xtraFormJavascriptIdentifier["_data" + xtraDataType + "Form"]._value = documentName;
    xtraFormJavascriptIdentifier["_data" + xtraDataType + "Form"]._xstore = xstoreName;
    xtraFormJavascriptIdentifier["_data" + xtraDataType + "Form"]._collection = collectionName;

    xtraFormJavascriptIdentifier.displayContextPanel();
}

function openPage(page, pageName, configuration)
{
    if (configuration == '' || configuration == null)
    {
        window.open(page, pageName, config='toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, location=no, status=yes');
    }
    else
    {
        window.open(page, pageName, config=configuration);
    }
}

/**
 * Start of Context menu functions.
 *
 * The following functions are all triggered by selecting options from the custom right click
 * context menus
 */

/**
 * Goes to the field details screen for the page that the context menu was triggered for.
 */
function menuViewDetails(obj)
{
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_form_map');

    document.result_form.presentation_xsl.value = 'field_details_html.xsl';

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    var forms = eval(obj)._forms;

    /*
     * Just show the first section for this page.
     * TODO: when multiple sections is implemented, we will probably want to give the user the choice.
     */
    var currentForm
    for(form in forms)
    {
        currentForm = form;
        break;
    }

    if (typeof(currentForm) == 'undefined')
    {
        alert('Please add a Section to this page before using this feature.');
    }
    else
    {
        formIDSetsSubmit(eval(obj)._name, currentForm);
    }
}

/**
 * Goes to the section structure screen for the page that the context menu was triggered for.
 */
function menuViewStructure(obj)
{
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_bindings_file');
    addOperation('load_form_map');

    document.result_form.presentation_xsl.value = 'form_structure_html.xsl';

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    var forms = eval(obj)._forms;

    /*
     * Just show the first section for this page.
     * TODO: when multiple sections is implemented, we will probably want to give the user the choice.
     */
    var currentForm
    for(form in forms)
    {
        currentForm = form;
        break;
    }

    if (typeof(currentForm) == 'undefined')
    {
        alert('Please add a Section to this page before using this feature.');
    }
    else
    {
        formIDSetsSubmit(eval(obj)._name, currentForm);
    }
}

/**
 * Goes to the bindings screen for the page that the context menu was triggered for.
 */
function menuViewBindings(obj)
{
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('load_form_structure');
    addOperation('load_bindings_file');
    addOperation('load_form_map');

    document.result_form.presentation_xsl.value = 'xpath_bindings_html.xsl';

    document.result_form.default_structure_type.value = 'Schema';
    document.result_form.default_structure_filename.value = '';

    var forms = eval(obj)._forms;

    /*
     * Just show the first section for this page.
     * TODO: when multiple sections is implemented, we will probably want to give the user the choice.
     */
    var currentForm
    for(form in forms)
    {
        currentForm = form;
        break;
    }

    if (typeof(currentForm) == 'undefined')
    {
        alert('Please add a Section to this page before using this feature.');
    }
    else
    {
        formIDSetsSubmit(eval(obj)._name, currentForm);
    }
}


function menuGeneratePage(obj)
{
    resetOperationsList()
    addOperation('save_form_map');
    checkServerProcessRequired();
    addOperation('generate_output');
    addOperation('load_form_map');

    document.result_form.presentation_xsl.value = 'form_map_html.xsl';

    var forms = eval(obj)._forms;

    /*
     * Just show the first section for this page.
     * TODO: when multiple sections is implemented, we will probably want to give the user the choice.
     */
    var currentForm
    for(form in forms)
    {
        currentForm = form;
        break;
    }

    if (typeof(currentForm) == 'undefined')
    {
        alert('Please add a Section to this page before using this feature.');
    }
    else
    {
        showStatusMessage("Generation in process...");
        formIDSetsSubmit(eval(obj)._name, currentForm);
    }
}

function menuPreviewPage(obj)
{
    if (typeof(navigationDiagram._preview._url) != 'undefined')
    {
        var pageName = eval(obj)._name;
        var previewUrl = navigationDiagram._preview._url + '/Page_Preview_' + pageName + '.html';

        var targetUrl = 'page_preview.html?page=' + pageName + '&url=' + previewUrl;
        WinManager.openNewChildWindowStart(targetUrl, "Page Preview", fm_project, pageName);

        WinManager.cancelPropagation(arguments.callee.caller.arguments[0]);
    }
    else
        alert('Please select a Preview Location before generating and previewing the page.');
}



/**
 * Returns the number of properties for the given object.
 */
function getPropertyCount(obj)
{
    var count = 0;
    for (prop in obj)
    {
        ++count;
    }
    return count;
}



/**
 * Called when the value of a link action is changed.
 * This checks to make sure the new value is not already in use.
 * If the current link (linkObj) is from a page, then we check against all other links.
 * If not, then we just check the new value against other links from a page.
 */
function checkActionNameUnique(field, linkObj)
{
    field.value = spacesToUnderscores(field.value);


    var unique = true;

    for(ln in navigationDiagram._links)
    {
        var link = navigationDiagram._links[ln];
        if (link != linkObj)
        {
            if ((link._source.constructor == Page) || (linkObj._source.constructor == Page))
            {
                if (link._action == field.value)
                {
                    unique = false;
                    break;
                }
            }
        }
    }

    if (unique)
    {
        if (linkObj._source.constructor == Page)
        {
            if (linkObj._action == '')
                navigationDiagram.storeActionAddition(field.value);
            else if (linkObj._action != field.value)
                navigationDiagram.storeActionRename(linkObj._action, field.value);
        }

        linkObj._action = field.value;
        linkObj.setChanged();

    }
    else
    {
        alert('Your action name is already in use.\n\nPlease choose another.');
        field.focus();
    }
    return unique
}

//Constants and mapping required for the palette drag and drop operations.
var constDRAG_HEIGHT = '20px';
var constDRAG_WIDTH = '100px';
var constDRAG_BG_COLOUR = "#FFFFBD";
var objControlNameMap = {add_page: 'full page', add_subsection: 'partial page', add_handler: 'controller', add_thirdPartyService: '3rd party service', add_link: 'action'}

/**
* Handles the mousedown event on the LHS palette.
* @param strControl Indicates the type of control selected on the palette - add_full_page|add_partial_page|...
* @param evt Event of interest is the mouse drag event.
*/
function handlePaletteControlDrag(evt, strControl)
{
    navigationDiagram.setMode(strControl);
    /* If nothing is selected on the diagram and an attempt is made to start linking then ensure selection of the start item. */
    if((typeof(navigationDiagram._selected) == 'undefined' || navigationDiagram._selected.constructor == Link) && strControl == 'add_link')
    {
        alert('Please select a component on the diagram to indicate the start of the link.');
        var objMouseMoveContainer = document.getElementById('mouse_move_container');
        if (objMouseMoveContainer != null)
        {
            hideComponent(objMouseMoveContainer);
        }
        navigationDiagram.setMode('select');
        return false;
    }

    var objMouseMoveContainer = document.getElementById('mouse_move_container');
    //Create the drag container if not present
    if (objMouseMoveContainer == null)
    {
        objMouseMoveContainer = document.createElement("div");
        objMouseMoveContainer.setAttribute("id", "mouse_move_container");
        objMouseMoveContainer.style.display = 'none';
        //Use a background iframe to ensure drag container is visible in the svg container.
        objMouseMoveTextBg = document.createElement('iframe');
        objMouseMoveText = document.createElement("div");
        objMouseMoveContainer.appendChild(objMouseMoveTextBg);
        objMouseMoveContainer.appendChild(objMouseMoveText);
        document.body.appendChild(objMouseMoveContainer);

        objMouseMoveContainer.style.position = "absolute";
        objMouseMoveContainer.style.width = constDRAG_WIDTH;
        objMouseMoveContainer.style.height = constDRAG_HEIGHT;

        objMouseMoveTextBg.style.width = "100%";
        objMouseMoveTextBg.style.height = constDRAG_HEIGHT;

        objMouseMoveText.style.position = 'absolute';
        objMouseMoveText.style.bottom = '0px';
        objMouseMoveText.style.left = '0px';
        objMouseMoveText.style.height = constDRAG_HEIGHT;
        objMouseMoveText.style.width = constDRAG_WIDTH;
        objMouseMoveText.style.backgroundColor = constDRAG_BG_COLOUR;
        objMouseMoveText.style.textAlign = 'center';
    }
    objMouseMoveText.innerHTML = objControlNameMap[strControl];
    objMouseMoveContainer.style.top = (evt.clientY + document.body.scrollTop) + 'px';
    objMouseMoveContainer.style.left = (Number(evt.clientX) + Number(10) + document.body.scrollLeft) + 'px';
    showComponent(objMouseMoveContainer);

    // cancel out any text selections
    document.body.focus();
    // prevent text selection in IE
    document.onselectstart = function () { return false; };
    // prevent IE from trying to drag an image
    objMouseMoveContainer.ondragstart = function() { return false; };
    // prevent text selection (except IE)
    return false;
}

//Add the mouse move and up events to the document. Note the svg container 'sits' on top.
document.onmousemove = fmMouseMove;
document.onmouseup = fmMouseUp;

var movingOverSVG = false;
/**
* Used to track the mouse pointer and create drag effect.
* @param e Event of interest is mouse move.
*/
function fmMouseMove(e)
{
    var evt = (e) ? e : window.event;
    var objMouseMoveContainer = document.getElementById('mouse_move_container');
    if (objMouseMoveContainer != null && !movingOverSVG)
    {
        //Track the mouse pointer.
        moveObjToCoords(objMouseMoveContainer, getMouseCoords(evt));
    }
}

/**
* Used to hide the drag container when the component has been dropped.
* @param e Event of interest is mouse up.
*/
function fmMouseUp(e)
{
    //alert('in doc mouse up');
    var evt = (e) ? e : window.event;
    var objMouseMoveContainer = document.getElementById('mouse_move_container');
    if (objMouseMoveContainer != null)
    {
        //hideComponent(objMouseMoveContainer);
        objMouseMoveContainer.parentNode.removeChild(objMouseMoveContainer);
        document.onselectstart = null;
        objMouseMoveContainer = null;
    }
    navigationDiagram.setMode('select');
    hideSVGContextMenu();
}

function overSVG()
{
    movingOverSVG = true;
}
function outSVG()
{
    movingOverSVG = false;
}

/**
 * Called when the mouse is being moved over the svg diagram.
 * @param evt The SVG event object for the move
 */
function svgMouseMove(evt)
{
    var objMouseMoveContainer = document.getElementById('mouse_move_container');
    if (objMouseMoveContainer != null)
    {
        var svgPos = getComponentPosition(navigationDiagram._embeddedSVG);
        moveObjToCoords(objMouseMoveContainer, {x: (evt.clientX + svgPos.x), y: (evt.clientY + svgPos.y)});
    }

    cancelEvent(evt);
    return false;
}


function hideSVGContextMenu(force)
{
    if (!movingOverSVG || force)
    {
        var menuDiv = document.getElementById('svg_context_menu');
        if (menuDiv != null)
        {
            hideComponent(menuDiv);
        }
    }
}

